@@ -4,7 +4,6 @@
 Command-line interface
 """
 
-import argparse
 import json
 import logging
 import os
@@ -14,8 +13,8 @@ from socket import gaierror
 
 from dataclasses import dataclass
 from typing import List, Dict, Union, Tuple, Any
-from urllib.parse import urlparse
 
+import toml
 import httpx
 from websockets.exceptions import WebSocketException
 
@@ -37,6 +36,13 @@ from speechmatics.models import (
     BatchSpeakerDiarizationConfig,
     RTSpeakerDiarizationConfig,
 )
+from speechmatics.cli_parser import (
+    parse_args,
+)
+from speechmatics.constants import (
+    BATCH_SELF_SERVICE_URL,
+    RT_SELF_SERVICE_URL,
+)
 
 LOGGER = logging.getLogger(__name__)
 
@@ -91,51 +97,6 @@ def parse_additional_vocab(additional_vocab_filepath):
     return additional_vocab
 
 
-def additional_vocab_item(to_parse):
-    """
-    Parses a single item of additional vocab. Used in conjunction with the
-    additional vocab command line argument.
-
-    :param to_parse: The item to parse.
-    :type to_parse: str
-
-    :return: Either a dictionary or a string depending on the form of the
-        additional vocab item.
-    :rtype: Union[dict, str]
-
-    :raises argparse.ArgumentTypeError: If the item to parse is invalid.
-    """
-    to_parse = str(to_parse)
-    parts = to_parse.split(":")
-    if len(parts) > 2:
-        raise argparse.ArgumentTypeError(
-            f"Can't have more than one separator (:) in additional vocab: "
-            f"{to_parse}."
-        )
-
-    content = parts[0]
-    if not content:
-        raise argparse.ArgumentTypeError(
-            f"Additional vocab must have content in: {to_parse}"
-        )
-
-    if len(parts) == 1:
-        return content
-
-    additional_vocab = {"content": content, "sounds_like": []}
-
-    sounds_likes = parts[1].split(",")
-    for sounds_like in sounds_likes:
-        if not sounds_like:
-            continue
-        additional_vocab["sounds_like"].append(sounds_like)
-
-    if not additional_vocab["sounds_like"]:
-        del additional_vocab["sounds_like"]
-
-    return additional_vocab
-
-
 def get_log_level(verbosity):
     """
     Returns the appropriate log level given a verbosity level.
@@ -165,7 +126,7 @@ class Transcripts:
     json: List[dict]
 
 
-def get_connection_settings(args):
+def get_connection_settings(args, lang="en"):
     """
     Helper function which returns a ConnectionSettings object based on the
     command line options given to the program.
@@ -176,10 +137,32 @@ def get_connection_settings(args):
     :return: Settings for the WebSocket connection.
     :rtype: speechmatics.models.ConnectionSettings
     """
+    auth_token = args.get("auth_token")
+    generate_temp_token = args.get("generate_temp_token")
+
+    home_directory = os.path.expanduser("~")
+    if os.path.exists(f"{home_directory}/.speechmatics/config"):
+        cli_config = {"default": {}}
+        with open(
+            f"{home_directory}/.speechmatics/config", "r", encoding="UTF-8"
+        ) as file:
+            cli_config = toml.load(file)
+        if cli_config["default"].get("auth_token") is not None and auth_token is None:
+            auth_token = cli_config["default"].get("auth_token", None)
+        if generate_temp_token is None:
+            generate_temp_token = cli_config["default"].get("generate_temp_token")
+
+    url = args.get("url", None)
+    if url is None:
+        if args.get("mode") == "batch":
+            url = BATCH_SELF_SERVICE_URL
+        else:
+            url = f"{RT_SELF_SERVICE_URL}/{lang}"
+
     settings = ConnectionSettings(
-        url=args.get("url"),
-        auth_token=args.get("auth_token"),
-        generate_temp_token=True if args.get("generate_temp_token", False) else None,
+        url=url,
+        auth_token=auth_token,
+        generate_temp_token=generate_temp_token,
     )
 
     if args.get("buffer_size") is not None:
@@ -441,6 +424,7 @@ def submit_job_and_wait(
 
 
 # pylint: disable=too-many-branches
+# pylint: disable=too-many-statements
 def main(args=None):
     """
     Main entrypoint.
@@ -469,13 +453,18 @@ def main(args=None):
                 LOGGER.error("No command specified")
                 args = vars(parse_args([mode, "-h"]))
             batch_main(args)
+        elif mode == "config":
+            if not args.get("command"):
+                LOGGER.error("No command specified")
+                args = vars(parse_args([mode, "-h"]))
+            config_main(args)
         else:
             # Not clear which help to show, so let's exit, but list the valid modes.
-            LOGGER.error("Usage: speechmatics [rt|batch] [command]")
+            LOGGER.error("Usage: speechmatics [rt|batch|config] [command]")
             raise SystemExit(
-                f"Unknown mode: {mode}, mode must be one of 'rt' (realtime) or 'batch'"
+                f"Unknown mode: {mode}, mode must be one of 'rt' (realtime), 'batch' or 'config'"
             )
-    except (KeyboardInterrupt, ValueError, TranscriptionError) as error:
+    except (KeyboardInterrupt, ValueError, TranscriptionError, KeyError) as error:
         LOGGER.info(error, exc_info=True)
         sys.exit(f"{type(error).__name__}: {error}")
     except FileNotFoundError as error:
@@ -518,14 +507,16 @@ def rt_main(args):
     :param args: arguments from parse_args()
     :type args: argparse.Namespace
     """
-    api = WebsocketClient(get_connection_settings(args))
+    transcription_config = get_transcription_config(args)
+    settings = get_connection_settings(args, lang=transcription_config.language)
+    api = WebsocketClient(settings)
 
-    if args["url"].lower().startswith("ws://") and args["ssl_mode"] != "none":
+    if settings.url.lower().startswith("ws://") and args["ssl_mode"] != "none":
         raise SystemExit(
             f"ssl_mode '{args['ssl_mode']}' is incompatible with"
             "protocol 'ws'. Use 'wss' instead."
         )
-    if args["url"].lower().startswith("wss://") and args["ssl_mode"] == "none":
+    if settings.url.lower().startswith("wss://") and args["ssl_mode"] == "none":
         raise SystemExit(
             "ssl_mode 'none' is incompatible with protocol 'wss'." "Use 'ws' instead."
         )
@@ -597,403 +588,64 @@ def batch_main(args):
             print(batch_client.check_job_status(args["job_id"]))
 
 
-# pylint: disable=too-many-locals
-# pylint: disable=too-many-statements
-def parse_args(args=None):
+def config_main(args):
+    """Main dispatch for "config" command set
+
+    :param args: arguments from parse_args()
+    :type args: argparse.Namespace
     """
-    Parses command-line arguments.
-
-    :param args: List of arguments to parse.
-    :type args: (List[str], optional)
-
-    :return: The set of arguments provided along with their values.
-    :rtype: Namespace
-    """
-
-    parser = argparse.ArgumentParser(description="CLI for Speechmatics products.")
-    parser.add_argument(
-        "-v",
-        dest="verbose",
-        action="count",
-        default=0,
-        help=(
-            "Set the log level for verbose logs. "
-            "The number of flags indicate the level, eg. "
-            "-v is INFO and -vv is DEBUG."
-        ),
-    )
-    parser.add_argument(
-        "--debug",
-        default=False,
-        action="store_true",
-        help=(
-            "Prints useful symbols to represent the messages on the wire. "
-            "Symbols are printed to STDERR, use only when STDOUT is "
-            "redirected to a file."
-        ),
-    )
-
-    # Parsers for shared arguments.
-    # Parent parser for commands involving files
-    file_parser = argparse.ArgumentParser(add_help=False)
-    file_parser.add_argument(
-        "files", metavar="FILEPATHS", type=str, nargs="+", help="File(s) to process."
-    )
-
-    # Parent parser for shared connection parameters
-    connection_parser = argparse.ArgumentParser(add_help=False)
-    connection_parser.add_argument(
-        "--url",
-        type=str,
-        required=True,
-        help=(
-            "Websocket for RT or for batch API URL (e.g. wss://192.168.8.12:9000/ "
-            "or https://trial.asr.api.speechmatics.com/v2 respectively)."
-        ),
-    )
-    connection_parser.add_argument(
-        "--auth-token",
-        type=str,
-        help=(
-            "Pre-shared authentication token to authorize the client."
-            "Applies to Speechmatics SaaS products."
-        ),
-    )
-    connection_parser.add_argument(
-        "--ssl-mode",
-        default="regular",
-        choices=["regular", "insecure", "none"],
-        help=(
-            "Use a preset configuration for the SSL context. With `regular` "
-            "mode a valid certificate is expected. With `insecure` mode"
-            " a self signed certificate is allowed."
-            " With `none` then SSL is not used."
-        ),
-    )
-    connection_parser.add_argument(
-        "--generate-temp-token",
-        default=None,
-        action="store_true",
-        help=(
-            "Automatically generate a temporary token for authentication."
-            "Non-enterprise customers must set this to True."
-            "Enterprise customers should set this to False."
-        ),
-    )
-
-    # Parent parser for shared params related to building a job config
-    config_parser = argparse.ArgumentParser(add_help=False)
-
-    config_parser.add_argument(
-        "--config-file",
-        dest="config_file",
-        type=str,
-        default=None,
-        help="Read the transcription config from a file."
-        " If you provide this, all other config options work as overrides.",
-    )
-    config_parser.add_argument(
-        "--lang",
-        "--language",
-        dest="language",
-        type=str,
-        default=None,
-        help="Language (ISO 639-1 code, e.g. en, fr, de).",
-    )
-    config_parser.add_argument(
-        "--operating-point",
-        choices=["standard", "enhanced"],
-        help=(
-            "Selects the acoustic model configuration. "
-            '"enhanced" is more computationally expensive than "standard" but '
-            "should result in a more accurate transcript."
-        ),
-    )
-    config_parser.add_argument(
-        "--domain",
-        type=str,
-        default=None,
-        help="Optionally request a specialized language pack, e.g. 'finance'",
-    )
-    config_parser.add_argument(
-        "--output-locale",
-        metavar="LOCALE",
-        type=str,
-        default=None,
-        help="Locale of the output of transcripts. eg. en-US.",
-    )
-    config_parser.add_argument(
-        "--additional-vocab",
-        nargs="*",
-        type=additional_vocab_item,
-        help=(
-            "Space separated list of additional vocab. Expected format: "
-            "<content (required)>:<sounds like (optional)>,<anymore sounds "
-            "like> Simple vocab list example: 'Speechmatics gnocchi'. "
-            "Vocab list with sounds like example: 'gnocchi:nokey,nochi'."
-        ),
-    )
-    config_parser.add_argument(
-        "--additional-vocab-file",
-        metavar="VOCAB_FILEPATH",
-        type=str,
-        help="File with additional vocab in JSON format.",
-    )
-    config_parser.add_argument(
-        "--punctuation-permitted-marks",
-        type=str,
-        default=None,
-        help=(
-            "Space separated list of permitted punctuation marks for advanced punctuation."
-        ),
-    )
-    config_parser.add_argument(
-        "--enable-entities",
-        default=False,
-        action="store_true",
-        help=(
-            "Whether to output additional information about recognised"
-            "entity classes (JSON output only)."
-        ),
-    )
-
-    # Parent parser for output type
-    output_format_parser = argparse.ArgumentParser(add_help=False)
-    output_format_parser.add_argument(
-        "--output-format",
-        default="txt",
-        choices=["txt", "json", "json-v2", "srt"],
-        type=str,
-        required=False,
-        help="Transcript output format.",
-    )
-
-    # Parent parser for batch diarization argument
-    batch_diarization_parser = argparse.ArgumentParser(add_help=False)
-    batch_diarization_parser.add_argument(
-        "--speaker-diarization-sensitivity",
-        type=float,
-        help="The sensitivity of the speaker detection. Default is 0.5 .",
-    )
-    batch_diarization_parser.add_argument(
-        "--diarization",
-        choices=["none", "speaker", "channel", "channel_and_speaker_change"],
-        help="Which type of diarization to use.",
-    )
-    batch_diarization_parser.add_argument(
-        "--channel-diarization-labels",
-        nargs="+",
-        help=(
-            "Add your own speaker or channel labels to the transcript. "
-            "Example usage: --channel-diarization-labels label1 label2"
-        ),
-    )
-
-    # Parent parser for job_id argument
-    job_id_parser = argparse.ArgumentParser(add_help=False)
-    job_id_parser.add_argument("--job-id", type=str, required=True, help="Job ID.")
-
-    # Parent parser for RT transcribe arguments
-    rt_transcribe_command_parser = argparse.ArgumentParser(add_help=False)
-    rt_transcribe_command_parser.add_argument(
-        "--enable-partials",
-        default=False,
-        action="store_true",
-        help=(
-            "Whether to return partial transcripts which can be updated by later,"
-            "final transcripts."
-        ),
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--punctuation-sensitivity",
-        type=float,
-        help="Sensitivity level for advanced punctuation.",
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--speaker-diarization-max-speakers",
-        type=int,
-        help="Enforces the maximum number of speakers allowed in a single audio stream. Min: 2, Max: 20, Default: 20.",
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--speaker-change-sensitivity",
-        type=float,
-        help="Sensitivity level for speaker change.",
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--speaker-change-token",
-        default=False,
-        action="store_true",
-        help="Shows a <sc> token where a speaker change was detected.",
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--max-delay",
-        type=float,
-        help="Maximum acceptable delay before sending a piece of transcript.",
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--max-delay-mode",
-        default="flexible",
-        choices=["fixed", "flexible"],
-        type=str,
-        help=(
-            "How to interpret the max-delay size if speech is in"
-            "the middle of an unbreakable entity like a number."
-        ),
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--raw",
-        metavar="ENCODING",
-        type=str,
-        help=(
-            "Indicate that the input audio is raw, provide the encoding"
-            "of this raw audio, eg. pcm_f32le."
-        ),
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--sample-rate",
-        type=int,
-        default=44_100,
-        help="The sample rate in Hz of the input audio, if in raw format.",
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--chunk-size",
-        type=int,
-        default=1024 * 4,
-        help=(
-            "How much audio data, in bytes, to send to the server in each "
-            "websocket message. Larger values can increase latency, but "
-            "values which are too small create unnecessary overhead."
-        ),
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--buffer-size",
-        default=512,
-        type=int,
-        help=(
-            "Maximum number of messages to send before waiting for"
-            "acknowledgements from the server."
-        ),
-    )
-    rt_transcribe_command_parser.add_argument(
-        "--print-json",
-        default=False,
-        action="store_true",
-        help=(
-            "Print the JSON partial & final transcripts received rather than "
-            "plaintext messages."
-        ),
-    )
-
-    rt_transcribe_command_parser.add_argument(
-        "--diarization",
-        choices=["none", "speaker", "speaker_change"],
-        help="Which type of diarization to use.",
-    )
-
-    # Build our actual parsers.
-    mode_subparsers = parser.add_subparsers(title="Mode", dest="mode")
-
-    # Parsers specific to rt mode commands.
-
-    rt_parser = mode_subparsers.add_parser("rt", help="Real-time commands")
-    rt_subparsers = rt_parser.add_subparsers(title="Commands", dest="command")
-    rt_subparsers.add_parser(
-        "transcribe",
-        parents=[
-            rt_transcribe_command_parser,
-            file_parser,
-            connection_parser,
-            config_parser,
-        ],
-    )
-
-    # Parsers specific to batch mode commands
-    batch_parser = mode_subparsers.add_parser("batch", help="Batch commands")
-    batch_subparsers = batch_parser.add_subparsers(title="Commands", dest="command")
-
-    batch_subparsers.add_parser(
-        "transcribe",
-        parents=[
-            file_parser,
-            connection_parser,
-            config_parser,
-            output_format_parser,
-            batch_diarization_parser,
-        ],
-        help="Transcribe one or more audio files using batch mode, while waiting for results.",
-    )
-
-    batch_subparsers.add_parser(
-        "submit",
-        parents=[
-            file_parser,
-            connection_parser,
-            config_parser,
-            output_format_parser,
-            batch_diarization_parser,
-        ],
-        help="Submit one or more files for transcription.",
-    )
-
-    batch_subparsers.add_parser(
-        "list-jobs",
-        parents=[connection_parser],
-        help="Retrieve json of last 100 jobs submitted within the last 7 days for the SaaS "
-        "or all of the jobs for the batch appliance",
-    )
-
-    batch_subparsers.add_parser(
-        "get-results",
-        parents=[connection_parser, output_format_parser, job_id_parser],
-        help="Retrieve results of a transcription job.",
-    )
-
-    batch_delete_parser = batch_subparsers.add_parser(
-        "delete",
-        parents=[connection_parser, output_format_parser, job_id_parser],
-        help="Delete the results of a transcription job.",
-    )
-    batch_delete_parser.add_argument(
-        "--force",
-        default=False,
-        action="store_true",
-        dest="force_delete",
-        help="Force deletion of a running job",
-    )
-
-    batch_subparsers.add_parser(
-        "job-status",
-        parents=[connection_parser, job_id_parser],
-        help="Retrieve status of a transcription job.",
-    )
-
-    # Parser for the "transcribe" command uses only RT.
-    mode_subparsers.add_parser(
-        "transcribe",
-        parents=[
-            rt_transcribe_command_parser,
-            file_parser,
-            connection_parser,
-            config_parser,
-        ],
-        help="Real-time commands. RETAINED FOR LEGACY COMPATIBILITY.",
-    )
-
-    parsed_args = parser.parse_args(args=args)
-
-    # Fix up args for transcribe command
-    if parsed_args.mode == "transcribe":
-        parsed_args.command = "transcribe"
-
-        if urlparse(parsed_args.url).scheme in ["ws", "wss"]:
-            parsed_args.mode = "rt"
+    home_directory = os.path.expanduser("~")
+    command = args.get("command")
+    if command == "set":
+        cli_config = {"default": {}}
+        if os.path.exists(f"{home_directory}/.speechmatics"):
+            if os.path.exists(f"{home_directory}/.speechmatics/config"):
+                with open(
+                    f"{home_directory}/.speechmatics/config", "r", encoding="UTF-8"
+                ) as file:
+                    toml_string = file.read()
+                    cli_config = toml.loads(toml_string)
         else:
-            LOGGER.error(
-                "speechmatics [transcribe] mode is used only with RT for legacy compatibility, not batch."
-            )
-            args = vars(parse_args(["batch", "-h"]))
+            os.makedirs(f"{home_directory}/.speechmatics")
 
-    return parsed_args
+        if args.get("auth_token"):
+            cli_config["default"]["auth_token"] = args.get("auth_token")
+        if args.get("generate_temp_token"):
+            cli_config["default"]["generate_temp_token"] = True
+
+        with open(
+            f"{home_directory}/.speechmatics/config", "w", encoding="UTF-8"
+        ) as file:
+            toml.dump(cli_config, file)
+
+    if command == "unset":
+        cli_config = {"default": {}}
+
+        if os.path.exists(f"{home_directory}/.speechmatics"):
+            if os.path.exists(f"{home_directory}/.speechmatics/config"):
+                with open(
+                    f"{home_directory}/.speechmatics/config", "r", encoding="UTF-8"
+                ) as file:
+                    toml_string = file.read()
+                    cli_config = toml.loads(toml_string)
+
+                if args.get("auth_token") and cli_config["default"].get("auth_token"):
+                    cli_config["default"].pop("auth_token")
+                if (
+                    args.get("generate_temp_token")
+                    and cli_config["default"].get("generate_temp_token") is not None
+                ):
+                    cli_config["default"].pop("generate_temp_token")
+
+                with open(
+                    f"{home_directory}/.speechmatics/config", "w", encoding="UTF-8"
+                ) as file:
+                    toml.dump(cli_config, file)
+                return
+
+        raise SystemExit(
+            f"Unable to remove config. No config file stored found at {home_directory}/.speechmatics/config"
+        )
 
 
 if __name__ == "__main__":
