@@ -5,8 +5,14 @@ import logging
 import os
 
 import pytest
+import toml
 
 from speechmatics import cli
+from speechmatics import cli_parser
+from speechmatics.constants import (
+    BATCH_SELF_SERVICE_URL,
+    RT_SELF_SERVICE_URL,
+)
 from tests.utils import path_to_test_resource
 
 
@@ -192,8 +198,8 @@ from tests.utils import path_to_test_resource
     ],
 )
 def test_cli_arg_parse_with_file(args, values):
-    connection_args = ["--url=example", "--auth-token=xyz"]
-    test_args = args + connection_args + ["fake_file.wav"]
+    common_transcribe_args = ["--auth-token=xyz", "--url=example", "fake_file.wav"]
+    test_args = args + common_transcribe_args
     actual_values = vars(cli.parse_args(args=test_args))
 
     for (key, val) in values.items():
@@ -351,16 +357,16 @@ def test_get_transcription_config_enable_partials(enable_partials, exp_value):
 
 
 def test_additional_vocab_item():
-    assert cli.additional_vocab_item("a") == "a"
-    assert cli.additional_vocab_item("a:") == {"content": "a"}
-    assert cli.additional_vocab_item("a:b,c") == {
+    assert cli_parser.additional_vocab_item("a") == "a"
+    assert cli_parser.additional_vocab_item("a:") == {"content": "a"}
+    assert cli_parser.additional_vocab_item("a:b,c") == {
         "content": "a",
         "sounds_like": ["b", "c"],
     }
     with pytest.raises(argparse.ArgumentTypeError):
-        cli.additional_vocab_item("")
+        cli_parser.additional_vocab_item("")
     with pytest.raises(argparse.ArgumentTypeError):
-        cli.additional_vocab_item("a:b:c")
+        cli_parser.additional_vocab_item("a:b:c")
 
 
 def test_get_log_level():
@@ -414,6 +420,32 @@ def test_rt_main_with_temp_token_option(mock_server):
     assert mock_server.path == "/v2"
 
 
+def test_rt_main_with_toml_config(mock_server):
+    args = [
+        "config",
+        "set",
+        "--auth-token=faketoken",
+    ]
+    cli.main(vars(cli.parse_args(args)))
+
+    args = [
+        "rt",
+        "transcribe",
+        "--ssl-mode=insecure",
+        "--url",
+        mock_server.url,
+        path_to_test_resource("ch.wav"),
+    ]
+    cli.main(vars(cli.parse_args(args)))
+    mock_server.wait_for_clean_disconnects()
+    print(mock_server.messages_received)
+    assert mock_server.clients_connected_count == 1
+    assert mock_server.clients_disconnected_count == 1
+    assert mock_server.messages_received
+    assert mock_server.messages_sent
+    assert mock_server.path == "/v2"
+
+
 def test_rt_main_with_all_options(mock_server, tmp_path):
     vocab_file = tmp_path / "vocab.json"
     vocab_file.write_text(
@@ -734,3 +766,103 @@ def test_add_printing_handlers_with_speaker_change_no_token(mocker, capsys):
         expected_transcript,
         speaker_change_token=False,
     )
+
+
+@pytest.mark.parametrize(
+    "args, values",
+    (
+        (
+            ["config", "set", "--auth-token=faketoken", "--generate-temp-token"],
+            {"auth_token": "faketoken", "generate_temp_token": True},
+        ),
+        (
+            ["config", "unset", "--auth-token", "--generate-temp-token"],
+            {"auth_token": True, "generate_temp_token": True},
+        ),
+    ),
+)
+def test_cli_argparse_config(args, values):
+
+    actual_values = vars(cli.parse_args(args=args))
+
+    for (key, val) in values.items():
+        assert actual_values[key] == val
+
+
+def test_config_set_toml():
+    args = {
+        "command": "set",
+        "auth_token": "faketoken",
+    }
+    try:
+        cli.config_main(args)
+    except Exception:  # pylint: disable=broad-except
+        assert False
+    home_dir = os.path.expanduser("~")
+    cli_config = {"default": {}}
+    with open(f"{home_dir}/.speechmatics/config", "r", encoding="UTF-8") as file:
+        cli_config = toml.load(file)
+
+    del args["command"]
+    for (key, val) in args.items():
+        assert cli_config["default"][key] == val
+
+
+@pytest.mark.parametrize(
+    "args",
+    (
+        {
+            "auth_token": "faketoken",
+            "generate_temp_token": True,
+        },
+        {
+            "auth_token": "faketoken",
+        },
+        {
+            "generate_temp_token": True,
+        },
+    ),
+)
+def test_config_set_and_remove_toml(args):
+    set_args = {"command": "set"}
+    set_args = {**set_args, **args}
+    try:
+        cli.config_main(set_args)
+    except Exception:  # pylint: disable=broad-except
+        assert False
+    home_dir = os.path.expanduser("~")
+    cli_config = {"default": {}}
+    with open(f"{home_dir}/.speechmatics/config", "r", encoding="UTF-8") as file:
+        cli_config = toml.load(file)
+
+    for (key, val) in args.items():
+        assert cli_config["default"][key] == val
+
+    unset_args = {"command": "unset"}
+    for key in ["auth_token", "generate_temp_token"]:
+        if key in args:
+            unset_args[key] = True
+        else:
+            unset_args[key] = False
+
+    try:
+        cli.config_main(unset_args)
+    except Exception:  # pylint: disable=broad-except
+        assert False
+    home_dir = os.path.expanduser("~")
+    cli_config = {"default": {}}
+    with open(f"{home_dir}/.speechmatics/config", "r", encoding="UTF-8") as file:
+        cli_config = toml.load(file)
+
+    for (key, val) in args.items():
+        assert key not in cli_config["default"]
+
+
+def test_default_urls_connection_config():
+    rt_args = {"mode": "rt"}
+    settings = cli.get_connection_settings(rt_args, lang="es")
+    assert settings.url == f"{RT_SELF_SERVICE_URL}/es"
+
+    batch_args = {"mode": "batch"}
+    settings = cli.get_connection_settings(batch_args, lang="es")
+    assert settings.url == f"{BATCH_SELF_SERVICE_URL}"
